@@ -26,6 +26,12 @@ def exists(val):
 def default(val, d):
     return val if exists(val) else d
 
+def find_first_index(cond, arr):
+    for ind, el in enumerate(arr):
+        if cond(el):
+            return ind
+    return -1
+
 def cast_list(val = None):
     if not exists(val):
         return []
@@ -78,7 +84,11 @@ def fetch_experiments_index(path):
 # fetch protein sequences by gene name and uniprot id
 
 class FactorProteinDatasetByUniprotID(Dataset):
-    def __init__(self, folder):
+    def __init__(
+        self,
+        folder,
+        species_priority = ['human', 'mouse']
+    ):
         super().__init__()
         fasta_paths = [*Path(folder).glob('*.fasta')]
         assert len(fasta_paths) > 0, f'no fasta files found at {folder}'
@@ -105,17 +115,42 @@ class FactorProteinDatasetByUniprotID(Dataset):
 # fetch
 
 class FactorProteinDataset(Dataset):
-    def __init__(self, folder, return_tuple_only = False):
+    def __init__(
+        self,
+        folder,
+        species_priority = ['human', 'mouse'],
+        return_tuple_only = False
+    ):
         super().__init__()
         fasta_paths = [*Path(folder).glob('*.fasta')]
         assert len(fasta_paths) > 0, f'no fasta files found at {folder}'
         self.paths = fasta_paths
-        self.index_by_gene = defaultdict(list)
+
+        index_by_gene = defaultdict(list)
         self.return_tuple_only = return_tuple_only # whether to return tuple even if there is only one subunit
 
         for path in fasta_paths:
             gene, uniprotid, *_ = path.stem.split('.')
-            self.index_by_gene[gene].append(path)
+            index_by_gene[gene].append(path)
+
+        # prioritize fasta files of certain species
+        # but allow for appropriate fallback, by order of species_priority
+
+        get_species_from_path = lambda p: p.stem.split('_')[1].lower()
+
+        filtered_index_by_gene = defaultdict(list)
+
+        for gene, gene_paths in index_by_gene.items():
+            species_count = list(map(lambda specie: len(list(filter(lambda p: get_species_from_path(p) == specie, gene_paths))), species_priority))
+            species_ind_non_zero = find_first_index(lambda t: t > 0, species_count)
+
+            if species_ind_non_zero == -1:
+                continue
+
+            species = species_priority[species_ind_non_zero]
+            filtered_index_by_gene[gene] = list(filter(lambda p: get_species_from_path(p) == species, gene_paths))
+
+        self.index_by_gene = filtered_index_by_gene
 
     def __len__(self):
         return len(self.paths)
@@ -178,6 +213,19 @@ def pl_notin(col, arr):
     equalities = list(map(lambda t: pl.col(col) != t, arr))
     return functools.reduce(lambda a, b: a & b, equalities)
 
+def filter_by_col_isin(df, col, arr, chunk_size = 25):
+    """
+    polars seem to have a bug
+    where OR more than 25 conditions freezes (for pl_isin)
+    do in chunks of 25 and then concat instead
+    """
+    dataframes = []
+    for i in range(0, len(arr), chunk_size):
+        sub_arr = arr[i:(i + chunk_size)]
+        filtered_df = df.filter(pl_isin(col, sub_arr))
+        dataframes.append(filtered_df)
+    return pl.concat(dataframes)
+
 def filter_bed_file_by_(bed_file_1, bed_file_2, output_file):
     # generated by OpenAI Codex
 
